package com.yaozhitech.spring5.config;

import java.util.Arrays;
import java.util.Map;

import org.springframework.beans.BeansException;
import org.springframework.context.ApplicationContext;
import org.springframework.context.ApplicationContextAware;
import org.springframework.context.ApplicationListener;
import org.springframework.context.EnvironmentAware;
import org.springframework.context.annotation.Configuration;
import org.springframework.context.event.ContextRefreshedEvent;
import org.springframework.core.env.Environment;

import lombok.extern.slf4j.Slf4j;

/**
 * 环境配置
 *
 * @author shouhua
 */
@Slf4j
@Configuration
public class EnvConfig implements ApplicationListener<ContextRefreshedEvent>, EnvironmentAware, ApplicationContextAware {

    private static Environment environment;

    private static ApplicationContext applicationContext;
    private volatile static Boolean ON_LINE;

    /**
     * 是否线上环境
     *
     * @return
     */
    public static boolean isOnLine() {
        if (ON_LINE == null) {
            ON_LINE = Arrays.asList(environment.getActiveProfiles()).contains(GlobalConstants.SPRING_PROFILE_PRODUCTION);
        }
        return ON_LINE;
    }

    /**
     * 获取当前环境
     *
     * @return
     */
    public static String getEnvironment() {
        if (Arrays.asList(environment.getActiveProfiles()).contains(GlobalConstants.SPRING_PROFILE_PRODUCTION)) {
            return GlobalConstants.SPRING_PROFILE_PRODUCTION;
        } else if (Arrays.asList(environment.getActiveProfiles()).contains(GlobalConstants.SPRING_PROFILE_DEVELOPMENT)) {
            return GlobalConstants.SPRING_PROFILE_DEVELOPMENT;
        } else if (Arrays.asList(environment.getActiveProfiles()).contains(GlobalConstants.SPRING_PROFILE_TEST)) {
            return GlobalConstants.SPRING_PROFILE_TEST;
        } else {
            return environment.getActiveProfiles().toString();
        }
    }

    /**
     * 获取配置
     *
     * @param key
     * @return
     */
    public static String getProperty(String key) {
        return environment.getProperty(key);
    }

    /**
     * 获取配置
     *
     * @param key
     * @param targetType
     * @param <T>
     * @return
     */
    public static <T> T getProperty(String key, Class<T> targetType) {
        return environment.getProperty(key, targetType);
    }

    public static <T> T getBean(Class<T> requiredType) {
        return EnvConfig.applicationContext.getBean(requiredType);
    }

    public static <T> Map<String, T> getBeansOfType(Class<T> requiredType) {
        return EnvConfig.applicationContext.getBeansOfType(requiredType);
    }

    public static ApplicationContext getApplicationContext() {
        return EnvConfig.applicationContext;
    }

    @Override
    public void onApplicationEvent(ContextRefreshedEvent event) {
        //刷新环境
        setApplicationContext(event.getApplicationContext());
        setEnvironment(EnvConfig.applicationContext.getEnvironment());
    }

    @Override
    public void setEnvironment(Environment environment) {
        EnvConfig.environment = environment;
        String active = EnvConfig.environment.getProperty("spring.profiles.active");
        log.warn("激活[{}]配置", active);
    }

    @Override
    public void setApplicationContext(ApplicationContext applicationContext) throws BeansException {
        EnvConfig.applicationContext = applicationContext;
    }
}
