package com.yaozhitech.spring5.controller;

import java.util.concurrent.TimeUnit;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.data.redis.core.ValueOperations;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.yaozhitech.spring5.domain.City;

import reactor.core.publisher.Mono;

@RestController
@RequestMapping(value = "/city")
public class CityWebFluxController {

	@Autowired
	private RedisTemplate<String, Object> redisTemplate;
	
	@GetMapping(value = "/{id}")
	public Mono<City> findCityById(@PathVariable("id") Long id) {
		String key = "city_" + id;
		ValueOperations<String, Object> operations = redisTemplate.opsForValue();
		boolean hasKey = redisTemplate.hasKey(key);
		City city = (City) operations.get(key);

		if (!hasKey) {
			return Mono.create(monoSink -> monoSink.success(null));
		}
		return Mono.create(monoSink -> monoSink.success(city));
	}

	@PostMapping()
	public Mono<City> saveCity(@RequestBody City city) {
		String key = "city_" + city.getId();
		ValueOperations<String, Object> operations = redisTemplate.opsForValue();
		operations.set(key, city, 160, TimeUnit.SECONDS);

		return Mono.create(monoSink -> monoSink.success(city));
	}

	@DeleteMapping(value = "/{id}")
	public Mono<Long> deleteCity(@PathVariable("id") Long id) {
		String key = "city_" + id;
		boolean hasKey = redisTemplate.hasKey(key);
		if (hasKey) {
			redisTemplate.delete(key);
		}
		return Mono.create(monoSink -> monoSink.success(id));
	}
}
