package com.yaozhitech.spring5.controller;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.ReactiveRedisTemplate;
import org.springframework.data.redis.core.ReactiveValueOperations;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.yaozhitech.spring5.domain.City;

import reactor.core.publisher.Mono;

@RestController
@RequestMapping(value = "/city2")
public class CityWebFluxReactiveController {

	@Autowired
	private ReactiveRedisTemplate reactiveRedisTemplate;

	@GetMapping(value = "/{id}")
	public Mono<City> findCityById(@PathVariable("id") Long id) {
		String key = "city_" + id;
		ReactiveValueOperations<String, City> operations = reactiveRedisTemplate.opsForValue();
		Mono<City> city = operations.get(key);
		return city;
	}

	@PostMapping
	public Mono<City> saveCity(@RequestBody City city) {
		String key = "city_" + city.getId();
		ReactiveValueOperations<String, City> operations = reactiveRedisTemplate.opsForValue();
		return operations.getAndSet(key, city);
	}

	@DeleteMapping(value = "/{id}")
	public Mono<Long> deleteCity(@PathVariable("id") Long id) {
		String key = "city_" + id;
		return reactiveRedisTemplate.delete(key);
	}
}
